SystemOrganization addCategory: #'Coral-Commandline'!

PPParser subclass: #CLIParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

CLIParser subclass: #CLIAbstractCommandParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIAbstractCommandParser commentStamp: 'DamienPollet 8/9/2011 17:34' prior: 0!
CLIAbstractCommand specifies and documents how to invoke a shell command or a sub-command thereof.
Concrete commands can be defined by subclassing, or by instanciating CLICommand.

Instance Variables:
	parser	<ProtoObject>!

!CLIAbstractCommandParser class methodsFor: 'testing' stamp: 'DamienPollet 8/10/2011 18:41'!
isAbstract
	^ self name == #CLIAbstractCommandParser! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/10/2011 23:52'!
action
	^ [:cmd | "do nothing" ]! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/9/2011 17:44'!
aliases
	"Synonyms or shortened versions of the main name"
	
	^ #()! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/20/2011 14:23'!
argumentsParser
	^ (self optionsParser , self parametersParser , self subcommandParser) end
		==> [ :nodes |
				self
					captureOptions: nodes first
					parameters: nodes second
					subcommand: nodes third ]! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/20/2011 15:27'!
asRootOn: argvArray in: environment 
	| context |
	context := self parseArguments: argvArray.
	context isPetitFailure
		ifTrue: [ CLIInvalidInvocation signalWith: argvArray ].
	
	^ context environment: environment! !

!CLIAbstractCommandParser methodsFor: 'capturing' stamp: 'DamienPollet 8/10/2011 19:27'!
captureOptions: optionsNodes parameters: parametersNodes subcommand: aSubcommand
	^ CLICommand
		from: self
		options: optionsNodes
		parameters: parametersNodes
		subcommand: aSubcommand! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/20/2011 13:46'!
commandParser
	^ (PPPredicateObjectParser anyOf: (self aliases copyWithFirst: self name)) token! !

!CLIAbstractCommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/9/2011 17:40'!
description
	"Verbose description of the command itself"
	
	^ ''! !

!CLIAbstractCommandParser methodsFor: 'testing' stamp: 'DamienPollet 8/11/2011 03:41'!
hasOptions
	^ self options notEmpty! !

!CLIAbstractCommandParser methodsFor: 'testing' stamp: 'DamienPollet 8/11/2011 03:41'!
hasParameters
	^ self parameters notEmpty! !

!CLIAbstractCommandParser methodsFor: 'testing' stamp: 'DamienPollet 8/11/2011 03:42'!
hasSubcommands
	^ self subcommands notEmpty! !

!CLIAbstractCommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/11/2011 03:39'!
help
	^ String streamContents: [ :help |
		help << 'Usage: ' << self synopsis; cr.
		help << self summary; cr
	]! !

!CLIAbstractCommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/9/2011 17:40'!
manual
	"Free-form detailed documentation, comes after summary, synopsis, description, and options"
	
	^ ''! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/9/2011 17:18'!
name
	"Canonic name"
	
	^ self subclassResponsibility! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/16/2011 18:56'!
on: argvArray in: environment 
	| context |
	context := self parse: argvArray.
	context isPetitFailure
		ifTrue: [ CLIInvalidInvocation signalWith: argvArray ].
	
	^ context environment: environment! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/9/2011 17:44'!
options
	"Optional arguments and flags"
	
	^ #()! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/11/2011 02:14'!
optionsParser
	^ self options
		ifEmpty: [ nil asParser ]
		ifNotEmptyDo: [ :opts | (PPChoiceParser withAll: opts) star ]! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/9/2011 17:44'!
parameters
	"Positional parameters"

	^ #()! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/11/2011 02:14'!
parametersParser
	^ self parameters
		ifEmpty: [ nil asParser ]
		ifNotEmptyDo: [ :params | PPSequenceParser withAll: params ]! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/20/2011 14:20'!
parse: anObject
	^ super parse:
		(anObject isString
			ifTrue: [ Character space split: anObject ]
			ifFalse: [ anObject ])
! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/20/2011 14:21'!
parseArguments: anObject
	^ self argumentsParser parse:
		(anObject isString
			ifTrue: [ Character space split: anObject ]
			ifFalse: [ anObject ])
! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/20/2011 14:23'!
parser
	"A parser that will recognize this command at the beginning of a stream of arguments"

	^ parser ifNil: [
		self commandParser , self argumentsParser
			==> [ :nodes | nodes second ] ]! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/20/2011 15:20'!
run
	"Run the command, for real (actual stdin/out/err, exit does exit).
	For testing inside an image, you want a CLIAdhocEnvironment instead"
	^ self
		runAsRootOn: (CommandLine uniqueInstance allArgumentsAfterOption: '--coral')
		in: CLIEnvironment new! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/20/2011 13:34'!
runAsRootOn: argvArray in: environment 
	^ (self asRootOn: argvArray in: environment) run! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 00:20'!
runIn: context
	"do nothing, should be redefined in subclasses"! !

!CLIAbstractCommandParser methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 00:30'!
runOn: argvArray in: environment 
	^ (self on: argvArray in: environment) run! !

!CLIAbstractCommandParser methodsFor: 'parsing' stamp: 'DamienPollet 8/11/2011 02:14'!
subcommandParser
	^ self subcommands
		ifEmpty: [ nil asParser ]
		ifNotEmptyDo: [ :subs | PPChoiceParser withAll: subs ]! !

!CLIAbstractCommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/9/2011 17:51'!
subcommands
	"Sub-commands, coming after or in place of any positional parameters"

	^ #()! !

!CLIAbstractCommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/9/2011 17:41'!
summary
	"One-line description of what this command is for"
	
	^ self subclassResponsibility! !

!CLIAbstractCommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/9/2011 17:50'!
synopsis
	"Typical patterns of invocation"
	
	^ String streamContents: [ :s |
		s nextPutAll: self name.
		self hasOptions ifTrue: [ s nextPutAll: ' [options...]' ].
		self hasParameters ifTrue: [ s nextPutAll: ' [arguments...]'].
		self hasSubcommands ifTrue: [ s nextPutAll: '[command]' ].
	]! !

CLIAbstractCommandParser subclass: #CLICommandParser
	instanceVariableNames: 'name aliases options parameters summary description subcommands action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

CLICommandParser class
	instanceVariableNames: 'subCommands'!

CLICommandParser class
	instanceVariableNames: 'subCommands'!

!CLICommandParser class methodsFor: 'testing' stamp: 'DamienPollet 8/9/2011 18:17'!
isAbstract
	^ false! !

!CLICommandParser class methodsFor: 'instance creation' stamp: 'DamienPollet 8/7/2011 23:53'!
named: aString
	^ self new setName: aString! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 16:15'!
addOption: anOption
	^ options add: anOption! !

!CLICommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/7/2011 23:50'!
aliases
	"Synonyms or shortened versions of the main name"
	
	^ aliases! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/7/2011 23:36'!
aliases: aCollection
	aliases := aCollection asArray! !

!CLICommandParser methodsFor: 'initialization-documentation' stamp: 'DamienPollet 8/9/2011 23:21'!
description: aString
	description := aString! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 16:28'!
do: aBlock
	action := aBlock! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 16:15'!
initialize
	super initialize.
	aliases := OrderedCollection new.
	options := OrderedCollection new.
	parameters := OrderedCollection new.
	subcommands := OrderedCollection new.! !

!CLICommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/7/2011 23:52'!
name
	"Canonic name"
	
	^ name! !

!CLICommandParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 18:34'!
option
	^ self addOption: CLIOptionParser new! !

!CLICommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/7/2011 23:50'!
options
	"Optional arguments and flags"

	^ options! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 20:28'!
options: aCollection
	options := aCollection.
	self resetParser.! !

!CLICommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/7/2011 23:50'!
parameters
	"Positional parameters"
	
	^ parameters! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 20:28'!
parameters: aCollection
	parameters := aCollection asArray.
	self resetParser.
	! !

!CLICommandParser methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 00:42'!
runIn: aContext
	action value: aContext! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/7/2011 23:53'!
setName: aString
	name := aString! !

!CLICommandParser methodsFor: 'accessing' stamp: 'DamienPollet 8/10/2011 16:15'!
subcommands
	"Sub-commands, coming after or in place of any positional parameters"
	
	^ subcommands! !

!CLICommandParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 20:28'!
subcommands: aCollection
	subcommands := aCollection asArray.
	self resetParser.! !

!CLICommandParser methodsFor: 'accessing-documentation' stamp: 'DamienPollet 8/11/2011 03:43'!
summary
	^ summary! !

!CLICommandParser methodsFor: 'initialization-documentation' stamp: 'DamienPollet 8/9/2011 23:20'!
summary: aString
	summary := aString! !

CLIParser subclass: #CLIOptionParser
	instanceVariableNames: 'short long parameter description action'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIOptionParser class methodsFor: 'testing' stamp: 'DamienPollet 8/9/2011 18:17'!
isAbstract
	^ false! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 15:57'!
- aCharacter
	self short: aCharacter! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 15:57'!
-- aString
	self long: aString! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 18:42'!
any
	self parameter: CLIParameterParser any! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 18:42'!
boolean
	self parameter: CLIParameterParser boolean! !

!CLIOptionParser methodsFor: 'capturing' stamp: 'DamienPollet 8/10/2011 19:00'!
capture
	^ CLIOption from: self! !

!CLIOptionParser methodsFor: 'capturing' stamp: 'DamienPollet 8/10/2011 19:46'!
capture: parameters
	^ CLIOption from: self parameter: parameters! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/10/2011 16:17'!
description
	^ description! !

!CLIOptionParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 16:17'!
description: aString
	description := aString! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 00:46'!
displayName
	| forms |
	forms := OrderedCollection new: 2.
	self shortForm ifNotNilDo: [ :it | forms add: it ].
	self longForm ifNotNilDo: [ :it | forms add: it ].
	^ $/ join: forms! !

!CLIOptionParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 16:26'!
do: aBlock
	action := aBlock! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 18:42'!
integer
	self parameter: CLIParameterParser integer! !

!CLIOptionParser methodsFor: 'initialization' stamp: 'DamienPollet 8/11/2011 01:04'!
long: aString
	aString isEmpty ifTrue: [ CLIInvalidSpec signal: 'Long form cannot be ''--''' ].
	long := aString asSymbol.
	self resetParser.! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:44'!
longForm
	"Verbose, double-dash form"
	^ long ifNotNil: [ '--' , long ]! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 00:46'!
name
	^ long ifNil: [short]! !

!CLIOptionParser methodsFor: 'initialization-convenience' stamp: 'DamienPollet 8/10/2011 18:42'!
number
	self parameter: CLIParameterParser number! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 15:04'!
parameter
	^ parameter! !

!CLIOptionParser methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 20:25'!
parameter: aParameter
	parameter := aParameter.
	self resetParser.! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/13/2011 17:41'!
parser
	^ parser ifNil: [ | choices |
		choices := OrderedCollection new: 2.
		self shortForm ifNotNilDo: [ :it | choices add: it asParser ].
		self longForm ifNotNilDo: [ :it | choices add: it asParser ].
		parser := PPElementParser
			on: (PPChoiceParser withAll: choices) token end
			message: 'Option ' , self name , ' expected'.
		parser := parameter isNil
			ifTrue: [ parser ==> [:node | self capture ] ]
			ifFalse: [ parser , parameter ==> [:nodes | self capture: nodes second ] ]
	]! !

!CLIOptionParser methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 03:03'!
runIn: commandContext option: optionContext
	action	ifNotNil: [ action cull: commandContext cull: optionContext ]! !

!CLIOptionParser methodsFor: 'initialization' stamp: 'DamienPollet 8/11/2011 01:05'!
short: aStringOrCharacter
	| name |
	name := aStringOrCharacter asString.
	name size ~= 1 ifTrue: [ CLIInvalidSpec signal: 'Short form must be a single character' ].
	name first == $- ifTrue: [ CLIInvalidSpec signal: 'Short form option cannot be --' ].
	short := name asSymbol.
	self resetParser.! !

!CLIOptionParser methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 01:04'!
shortForm
	"Terse, clusterable form (dash-character)"
	^ short ifNotNil: [ '-' , short ]! !

CLIParser subclass: #CLIParameterParser
	instanceVariableNames: 'name isOptional isMultiple'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIParameterParser commentStamp: 'DamienPollet 8/9/2011 17:37' prior: 0!
CLIParameter defines positional parameters to either a command or an option.
Parameters with >= multiplicities can be defined by having the parser match more than one ARGV element.

Instance Variables:
	name	<ProtoObject | PseudoContext>
	isOptional	<Boolean>
	isMultiple	<Boolean>
	parser	<PPParser>!

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/9/2011 19:02'!
any
	^ self new
		setParser: #any asParser;
		name: 'param'
		! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/9/2011 20:56'!
anyOf: anArray
	^ self new
		setParser: (PPPredicateObjectParser anyOf: anArray);
		name: ('|' join: anArray)! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/13/2011 17:57'!
boolean
	| trueToken falseToken |
	trueToken := (self caseInsensitiveKeywords: #('true' 'yes')) end ==> [:token | true].
	falseToken := (self caseInsensitiveKeywords: #('false' 'no')) end ==> [:token | false].
	^ self new
		setParser: (PPElementParser on: trueToken / falseToken message: 'boolean expected');
		name: 'bool'! !

!CLIParameterParser class methodsFor: 'parsers' stamp: 'DamienPollet 8/9/2011 18:54'!
caseInsensitiveKeywords: aCollection
	^ (PPChoiceParser withAll: (aCollection collect: [ :each | each asParser caseInsensitive ])) token! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/13/2011 17:58'!
integer
	^ self new setParser:
		(PPElementParser
				on: #digit asParser plus token end ==> [ :token | token value asNumber ]
				message: 'positive integer expected');
			name: 'num'
		! !

!CLIParameterParser class methodsFor: 'testing' stamp: 'DamienPollet 8/9/2011 18:18'!
isAbstract
	^ false! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/4/2011 13:04'!
keyword: aString
	^ self new
		setParser: (PPPredicateObjectParser expect: aString message: aString , ' expected');
		name: aString
		! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/13/2011 17:59'!
number
	^ self new setParser:
		(PPElementParser
				on: ($- asParser optional , #digit asParser plus , ($. asParser , #digit asParser plus) optional) token end
						==> [ :token | token value asNumber ]
				message: 'number expected');
			name: 'num'
		! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/4/2011 13:17'!
path
	self shouldBeImplemented.
	"^ self new
		setParser: (PPPredicateObjectParser matching: FilesystemPathParser message: 'path expected');
		name: 'path'
	"! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/13/2011 18:00'!
relativeInteger
	^ self new setParser:
		(PPElementParser
				on: ($- asParser optional , #digit asParser plus) token end ==> [ :token | token value asNumber ]
				message: 'relative integer expected');
			name: 'num'
		! !

!CLIParameterParser class methodsFor: 'factory' stamp: 'DamienPollet 8/4/2011 13:16'!
url
	self shouldBeImplemented.
	"^ self new
		setParser: (PPPredicateObjectParser matching: urlParser message: 'URL expected');
		name: 'url'
	"! !

!CLIParameterParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 10:19'!
description
	^ self shouldBeImplemented! !

!CLIParameterParser methodsFor: 'initialization' stamp: 'DamienPollet 8/4/2011 13:14'!
initialize
	isOptional := isMultiple := false! !

!CLIParameterParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 17:06'!
name
	"Name of this parameter in the synopsis. Should reveal its type, role, and multiplicity."
	^ name! !

!CLIParameterParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 17:18'!
name: aString
	"Name of this parameter in the synopsis. Should reveal its type, role, and multiplicity.
	
	Brackets and punctuation often denote this:
		foo			keyword (typed as is)
		<foo>		meta-syntactic
		foo...		several items of the foo kind
		[foo|bar]	either one
		[foo]		optionally a foo
		[foo...]		zero or more foos"
	
	name := aString! !

!CLIParameterParser methodsFor: 'initialization' stamp: 'DamienPollet 8/4/2011 13:13'!
optional
	isOptional := true.
	parser := parser optional! !

!CLIParameterParser methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 11:48'!
parser
	^ parser! !

!CLIParameterParser methodsFor: 'initialization' stamp: 'DamienPollet 8/4/2011 13:13'!
plus
	isMultiple := true.
	parser := parser plus! !

!CLIParameterParser methodsFor: 'initialization' stamp: 'DamienPollet 8/4/2011 11:47'!
setParser: aParser
	parser := aParser! !

!CLIParameterParser methodsFor: 'initialization' stamp: 'DamienPollet 8/4/2011 13:13'!
star
	isOptional := isMultiple := true.
	parser := parser star! !

!CLIParser class methodsFor: 'testing' stamp: 'DamienPollet 8/9/2011 18:17'!
isAbstract
	^ true! !

!CLIParser methodsFor: 'parsing' stamp: 'DamienPollet 8/10/2011 17:18'!
parse: anObject
	^ super parse:
		(anObject isString
			ifTrue: [ Character space split: anObject ]
			ifFalse: [ anObject ])
! !

!CLIParser methodsFor: 'parsing' stamp: 'DamienPollet 8/10/2011 17:16'!
parseOn: aStream
	^ self parser parseOn: aStream! !

!CLIParser methodsFor: 'parsing' stamp: 'DamienPollet 8/10/2011 20:27'!
resetParser
	parser := nil! !

Exception subclass: #CLIExit
	instanceVariableNames: 'exitStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIExit class methodsFor: 'signaling' stamp: 'DamienPollet 8/11/2011 03:55'!
signalWith: anInteger
	self new
		exitStatus: anInteger;
		signal! !

!CLIExit methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:55'!
exitStatus
	^ exitStatus! !

!CLIExit methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:56'!
exitStatus: anInteger
	exitStatus := anInteger! !

!PPPredicateObjectParser class methodsFor: '*Coral-Commandline' stamp: 'DamienPollet 8/13/2011 03:24'!
matching: elementParser
	^ self
		matching: elementParser
		message: 'object matching ' , elementParser printString , ' expected'! !

!PPPredicateObjectParser class methodsFor: '*Coral-Commandline' stamp: 'DamienPollet 8/13/2011 17:31'!
matching: elementParser message: aString
	| result |
	^ self any ==> [:node | elementParser parse: node]
! !

PPDelegateParser subclass: #PPElementParser
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!PPElementParser class methodsFor: 'instance creation' stamp: 'DamienPollet 8/9/2011 20:47'!
on: aParser message: aString
	^ (self on: aParser) setMessage: aString! !

!PPElementParser methodsFor: 'parsing' stamp: 'DamienPollet 8/9/2011 20:53'!
parseOn: aStream
	| result |
	aStream atEnd ifTrue: [ ^ PPFailure message: 'element expected' at: aStream position ].
	result := parser "end" parse: aStream uncheckedPeek.
	^ result isPetitFailure
		ifFalse: [
			aStream next.
			result ]
		ifTrue: [
			PPFailure
				message: (message ifNil: ['error: ' , result printString , ' in element'])
				at: aStream position ]
! !

!PPElementParser methodsFor: 'initialization' stamp: 'DamienPollet 8/9/2011 20:48'!
setMessage: aString
	message := aString! !

Error subclass: #CLIInvalidInvocation
	instanceVariableNames: 'invocation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIInvalidInvocation class methodsFor: 'signaling' stamp: 'DamienPollet 8/16/2011 18:56'!
signalWith: anObject
	^ self new invocation: anObject; signal! !

!CLIInvalidInvocation methodsFor: 'accessing' stamp: 'DamienPollet 8/16/2011 19:13'!
description
	^ super description ,
		'(' , (Character space join: invocation) , ')'! !

!CLIInvalidInvocation methodsFor: 'initialization' stamp: 'DamienPollet 8/16/2011 19:04'!
initialize
	invocation := #()! !

!CLIInvalidInvocation methodsFor: 'initialization' stamp: 'DamienPollet 8/16/2011 18:56'!
invocation: anObject
	invocation := anObject! !

Error subclass: #CLIInvalidSpec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

Object subclass: #CLICommand
	instanceVariableNames: 'specification options parameters subcommand environment exitStatus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLICommand class methodsFor: 'instance creation' stamp: 'DamienPollet 8/10/2011 19:25'!
from: specification options: optionsArray parameters: parametersArray subcommand: aSubcommand 
	^ self basicNew
		initializeWith: specification
		options: optionsArray
		parameters: parametersArray
		subcommand: aSubcommand! !

!CLICommand methodsFor: 'input-output' stamp: 'DamienPollet 8/11/2011 01:18'!
<< aString 
	^ environment out << aString! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:11'!
atOption: anOptionParser
	^ self atOption: anOptionParser ifNone: [ NotFound signalFor: anOptionParser ]! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:16'!
atOption: anOptionParser ifNone: aBlock
	| opt |
	opt := options
				detect: [:o | o specification == anOptionParser ]
				ifNone: [ ^ aBlock value ].
	^ opt parameter! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 01:15'!
atOptionNamed: aSymbol
	^ self atOptionNamed: aSymbol ifNone: [ NotFound signalFor: aSymbol ]! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:17'!
atOptionNamed: aSymbol ifNone: aBlock
	| opt |
	opt := options
				detect: [:o | o name == aSymbol ]
				ifNone: [ ^ aBlock value ].
	^ opt parameter! !

!CLICommand methodsFor: 'input-output' stamp: 'DamienPollet 8/11/2011 03:18'!
cr
	^ environment out cr! !

!CLICommand methodsFor: 'initialization' stamp: 'DamienPollet 8/11/2011 00:03'!
environment: anEnvironment 
	environment := anEnvironment! !

!CLICommand methodsFor: 'input-output' stamp: 'DamienPollet 8/11/2011 03:28'!
err
	^ environment err! !

!CLICommand methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 03:46'!
exitStatus: anInteger 
	exitStatus := anInteger! !

!CLICommand methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 03:47'!
exitSuccess
	self signalExit: 0! !

!CLICommand methodsFor: 'testing' stamp: 'DamienPollet 8/11/2011 01:26'!
hasOptionNamed: aSymbol 
	self atOptionNamed: aSymbol ifNone: [ ^ false ].
	^ true! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 03:29'!
help
	^ specification help! !

!CLICommand methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 19:29'!
initializeWith: aCommandParser options: optionsArray parameters: parametersArray subcommand: aSubcommand 
	specification := aCommandParser.
	options := optionsArray.
	parameters := parametersArray.
	subcommand := aSubcommand! !

!CLICommand methodsFor: 'input-output' stamp: 'DamienPollet 8/20/2011 15:30'!
newline
	^ environment out newline! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 01:54'!
options
	^ options ifNil: #()! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 01:54'!
parameters
	^ parameters ifNil: #()! !

!CLICommand methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 04:06'!
run
	[
		options do: [ :each | each runIn: self ].
		specification runIn: self
	]
	on: CLIExit do: [ :exit |
		self exitStatus: exit exitStatus.
		environment exitWithStatus: exitStatus ]! !

!CLICommand methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 03:53'!
signalExit: anInteger
	CLIExit signalWith: anInteger! !

!CLICommand methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 01:54'!
subcommand
	^ subcommand! !

Object subclass: #CLIEnvironment
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

CLIEnvironment subclass: #CLIAdhocEnvironment
	instanceVariableNames: 'in out err'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIAdhocEnvironment class methodsFor: 'instance creation' stamp: 'DamienPollet 8/10/2011 17:59'!
in: stdinStream out: stdoutStream err: stderrStream
	^ self new
		in: stdinStream out: stdoutStream err: stderrStream
! !

!CLIAdhocEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/16/2011 17:46'!
err
	^ err! !

!CLIAdhocEnvironment methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 04:07'!
exitWithStatus: anInteger 
	"Do nothing. A real environment would quit the image etc."! !

!CLIAdhocEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/16/2011 17:46'!
in
	^ in! !

!CLIAdhocEnvironment methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 18:00'!
in: stdinStream out: stdoutStream err: stderrStream 
	in := stdinStream.
	out := stdoutStream.
	err := stderrStream! !

!CLIAdhocEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/16/2011 17:46'!
out
	^ out! !

!CLIEnvironment methodsFor: 'convenience' stamp: 'DamienPollet 8/16/2011 18:05'!
<< anObject
	^ self out << anObject! !

!CLIEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/20/2011 15:01'!
err
	^ OSProcess thisOSProcess stdErr "FileStream stderr"! !

!CLIEnvironment methodsFor: 'running' stamp: 'DamienPollet 8/16/2011 17:53'!
exitWithStatus: anInteger 
	"Quit the VM process, returning anInteger to the parent process."
	
	self flag: #vmDoesNotKnowHowToProperlyExit.
	SmalltalkImage current snapshot: false andQuit: true! !

!CLIEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/20/2011 15:00'!
in
	^ OSProcess thisOSProcess stdIn "FileStream stdout"! !

!CLIEnvironment methodsFor: 'accessing' stamp: 'DamienPollet 8/20/2011 15:00'!
out
	^ OSProcess thisOSProcess stdOut "FileStream stdout"! !

Object subclass: #CLIOption
	instanceVariableNames: 'specification parameter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Coral-Commandline'!

!CLIOption class methodsFor: 'instance creation' stamp: 'DamienPollet 8/10/2011 19:02'!
from: anOptionParser
	^ self basicNew initializeWith: anOptionParser! !

!CLIOption class methodsFor: 'instance creation' stamp: 'DamienPollet 8/10/2011 19:47'!
from: anOptionParser parameter: anObject
	^ self basicNew initializeWith: anOptionParser parameter: anObject! !

!CLIOption methodsFor: 'comparing' stamp: 'DamienPollet 8/11/2011 02:29'!
= anObject
	(anObject isKindOf: self class) ifFalse: [ ^ false ].
	^ specification = anObject specification
		and: [ parameter = anObject parameter ]! !

!CLIOption methodsFor: 'comparing' stamp: 'DamienPollet 8/11/2011 02:29'!
hash
	^ (self species hash bitXor: specification hash) bitXor: parameter hash! !

!CLIOption methodsFor: 'initialization' stamp: 'DamienPollet 8/10/2011 19:47'!
initializeWith: anOptionParser
	self initializeWith: anOptionParser parameter: nil! !

!CLIOption methodsFor: 'initialization' stamp: 'DamienPollet 8/11/2011 02:29'!
initializeWith: anOptionParser parameter: anObject
	specification := anOptionParser.
	parameter := anObject! !

!CLIOption methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 02:01'!
name
	^ specification name! !

!CLIOption methodsFor: 'accessing' stamp: 'DamienPollet 8/11/2011 02:30'!
parameter
	^ parameter! !

!CLIOption methodsFor: 'running' stamp: 'DamienPollet 8/11/2011 03:04'!
runIn: aCommand
	specification runIn: aCommand option: self! !

!CLIOption methodsFor: 'accessing' stamp: 'DamienPollet 8/10/2011 19:37'!
specification
	^ specification! !
